import anthropic # Anthropic's official Python library
import logging
# Updated import path for LlmServiceBase
from src.e_Infra.g_McpInfra.b_LlmManager.LlmServiceBase import LlmServiceBase

logger = logging.getLogger(__name__)

class AnthropicService(LlmServiceBase):
    """
    A service class to encapsulate interactions with the Anthropic API,
    adhering to the LlmServiceBase interface.
    """
    DEFAULT_MODEL_NAME = "claude-3-haiku-20240307" # As per new defaults
    DEFAULT_TEMPERATURE = 0.7                      # As per new defaults
    DEFAULT_MAX_OUTPUT_TOKENS = 2048               # As per new defaults (Anthropic uses max_tokens_to_sample or max_tokens in messages.create)

    def __init__(self, api_key: str, model_name: str = None, temperature: float = None, max_output_tokens: int = None):
        """
        Initializes the AnthropicService.

        Args:
            api_key (str): The Anthropic API key.
            model_name (str, optional): Specific Anthropic model. Defaults to self.DEFAULT_MODEL_NAME.
            temperature (float, optional): Sampling temperature. Defaults to self.DEFAULT_TEMPERATURE.
            max_output_tokens (int, optional): Max tokens for response. Defaults to self.DEFAULT_MAX_OUTPUT_TOKENS.
        Raises:
            ValueError: If API key is not provided or params are invalid.
            RuntimeError: If Anthropic client initialization fails.
        """
        if not api_key:
            logger.error("API key not provided for AnthropicService initialization.")
            raise ValueError("API key for Anthropic must be provided.")

        self.api_key = api_key
        self.model_name = model_name or self.DEFAULT_MODEL_NAME

        if temperature is not None:
            try:
                self.temperature = float(temperature)
                if not (0.0 <= self.temperature <= 1.0): # Anthropic typical range
                     logger.warning(f"Temperature {self.temperature} for Anthropic is outside typical range (0.0-1.0). Using it anyway.")
            except ValueError:
                logger.error(f"Invalid temperature value '{temperature}'. Must be a float. Using default {self.DEFAULT_TEMPERATURE}.")
                self.temperature = self.DEFAULT_TEMPERATURE
        else:
            self.temperature = self.DEFAULT_TEMPERATURE

        if max_output_tokens is not None:
            try:
                self.max_output_tokens = int(max_output_tokens)
                if self.max_output_tokens <= 0:
                    logger.warning(f"Invalid max_output_tokens {self.max_output_tokens}. Must be positive. Using default {self.DEFAULT_MAX_OUTPUT_TOKENS}.")
                    self.max_output_tokens = self.DEFAULT_MAX_OUTPUT_TOKENS
            except ValueError:
                logger.error(f"Invalid max_output_tokens value '{max_output_tokens}'. Must be an int. Using default {self.DEFAULT_MAX_OUTPUT_TOKENS}.")
                self.max_output_tokens = self.DEFAULT_MAX_OUTPUT_TOKENS
        else:
            self.max_output_tokens = self.DEFAULT_MAX_OUTPUT_TOKENS

        try:
            self.client = anthropic.Anthropic(api_key=self.api_key)
            logger.info(f"AnthropicService initialized: model='{self.model_name}', temperature={self.temperature}, max_output_tokens={self.max_output_tokens}.")
        except Exception as e:
            logger.error(f"Error during AnthropicService init (model: {self.model_name}): {e}", exc_info=True)
            raise RuntimeError(f"Failed to initialize Anthropic Service (model: {self.model_name}): {e}")

    def generate_text(self, prompt: str) -> str:
        """
        Generates text using the configured Anthropic model.

        Args:
            prompt (str): The prompt to send to the Anthropic API.
                          This will be used as the content of a user message.

        Returns:
            str: The text response generated by the Anthropic API.

        Raises:
            RuntimeError: If there's an error during text generation.
        """
        if not prompt:
            logger.warning("Generate_text called with an empty prompt.")
            return ""

        try:
            message = self.client.messages.create(
                model=self.model_name,
                max_tokens=self.max_output_tokens,
                temperature=self.temperature,
                messages=[
                    {
                        "role": "user",
                        "content": prompt
                    }
                ]
            )
            if message.content and isinstance(message.content, list) and len(message.content) > 0:
                if hasattr(message.content[0], 'text'):
                    return message.content[0].text.strip()
                else:
                    logger.error("Anthropic API returned a content block of unexpected type or without text.")
                    raise RuntimeError("Anthropic API returned no text content in the expected block type.")
            else:
                logger.error("Anthropic API returned an unexpected response structure or empty content.")
                raise RuntimeError("Anthropic API returned no content.")
        except anthropic.APIError as e:
            logger.error(f"Anthropic API error during text generation (model: {self.model_name}): {e}", exc_info=True)
            raise RuntimeError(f"Failed to generate text using Anthropic (model: {self.model_name}): {e}")
        except Exception as e:
            logger.error(f"Unexpected error during Anthropic text generation (model: {self.model_name}): {e}", exc_info=True)
            raise RuntimeError(f"An unexpected error occurred with Anthropic (model: {self.model_name}): {e}")

    def check_connection(self) -> bool:
        """
        Performs a lightweight check for the Anthropic client.
        """
        try:
            self.client.messages.create(
                model=self.model_name,
                max_tokens=10, # Minimal tokens for a health check
                temperature=self.temperature, # Use configured temperature
                messages=[{"role": "user", "content": "Health check."}]
            )
            logger.info("Anthropic API connection check successful (via minimal generation).")
            return True
        except anthropic.APIError as e:
            if isinstance(e, anthropic.AuthenticationError):
                 logger.error(f"Anthropic API connection check failed: AuthenticationError: {e}", exc_info=True)
            else:
                 logger.error(f"Anthropic API connection check failed: APIError: {e}", exc_info=True)
            return False
        except Exception as e:
            logger.error(f"Unexpected error during Anthropic connection check: {e}", exc_info=True)
            return False
